@@ -123,15 +123,15 @@ sub __is_NonNative {
 	my $cf = ((split(/\//,$file))[-1]);
 	my @songa = pss($metastuff->{_TRACKNUM});
 	
-	$rh{artist}   = getutf8($metastuff->{_ARTIST} || "Unknown Artist");
-	$rh{album}    = getutf8($metastuff->{_ALBUM}  || "Unknown Album");
-	$rh{title}    = getutf8($metastuff->{_TITLE}  || $cf || "Unknown Title");
-	$rh{genre}    = getutf8($metastuff->{_GENRE}  || "");
-	$rh{songs}    = int($songa[1]);
-	$rh{songnum}  = int($songa[0]); 
-	$rh{comment}  = getutf8($metastuff->{_COMMENT} || $metastuff->{FORMAT}." file");
-	$rh{fdesc}    = getutf8($metastuff->{_VENDOR} || "Converted using $encoder"); 
-	$rh{mediatype}= int($metastuff->{_MEDIATYPE} || MEDIATYPE_AUDIO);
+	$rh{artist}    = getutf8($metastuff->{_ARTIST} || "Unknown Artist");
+	$rh{album}     = getutf8($metastuff->{_ALBUM}  || "Unknown Album");
+	$rh{title}     = getutf8($metastuff->{_TITLE}  || $cf || "Unknown Title");
+	$rh{genre}     = getutf8($metastuff->{_GENRE}  || "");
+	$rh{songs}     = int($songa[1]);
+	$rh{songnum}   = int($songa[0]); 
+	$rh{comment}   = getutf8($metastuff->{_COMMENT} || $metastuff->{FORMAT}." file");
+	$rh{fdesc}     = getutf8($metastuff->{_VENDOR}  || "Converted using $encoder"); 
+	$rh{mediatype} = int($metastuff->{_MEDIATYPE}   || MEDIATYPE_AUDIO);
 	return {ref=>\%rh, encoder=>$encoder, codec=>$NN_HEADERS->{$magic}->{ftyp} };
 }
 
@@ -154,21 +154,21 @@ sub __is_qt {
  }
  
  my $cf = ((split(/\//,$file))[-1]);
- $rh{songs}     = int($ret->{tracks});
- $rh{songnum}   = int($ret->{tracknum});
- $rh{cds}       = int($ret->{cds});
- $rh{cdnum}     = int($ret->{cdnum});
- $rh{srate}     = int($ret->{srate});
- $rh{time}      = int($ret->{time});
- $rh{bitrate}   = int($ret->{bitrate});
- $rh{filesize}  = int($ret->{filesize});
- $rh{fdesc}     = getutf8($ret->{fdesc});
- $rh{artist}    = getutf8($ret->{artist}   || "Unknown Artist");
- $rh{album}     = getutf8($ret->{album}    || "Unknown Album");
- $rh{title}     = getutf8($ret->{title}    || $cf || "Unknown Title");
- $rh{genre}     = _get_genre( getutf8($ret->{genre} || $ret->{gnre} || "") );
- $rh{composer}  = getutf8($ret->{composer} || ""); 
- $rh{soundcheck}= _parse_iTunNORM($ret->{iTunNORM});
+ $rh{songs}      = int($ret->{tracks});
+ $rh{songnum}    = int($ret->{tracknum});
+ $rh{cds}        = int($ret->{cds});
+ $rh{cdnum}      = int($ret->{cdnum});
+ $rh{srate}      = int($ret->{srate});
+ $rh{time}       = int($ret->{time});
+ $rh{bitrate}    = int($ret->{bitrate});
+ $rh{filesize}   = int($ret->{filesize});
+ $rh{fdesc}      = getutf8($ret->{fdesc});
+ $rh{artist}     = getutf8($ret->{artist}   || "Unknown Artist");
+ $rh{album}      = getutf8($ret->{album}    || "Unknown Album");
+ $rh{title}      = getutf8($ret->{title}    || $cf || "Unknown Title");
+ $rh{genre}      = _get_genre( getutf8($ret->{genre} || $ret->{gnre} || "") );
+ $rh{composer}   = getutf8($ret->{composer} || ""); 
+ $rh{soundcheck} = _parse_iTunNORM($ret->{iTunNORM});
  $rh{mediatype}  = int($ret->{mediatype} || MEDIATYPE_AUDIO);
  return  ({codec=>$ret->{_CODEC}, ref=>\%rh});
 }
@@ -245,13 +245,13 @@ sub __is_mp3 {
 	my $cf = ((split(/\//,$file))[-1]);
 	
 	my %rh = ();
-	$rh{bitrate} = $h->{BITRATE};
+	$rh{bitrate}  = $h->{BITRATE};
 	$rh{filesize} = $h->{SIZE};
 	$rh{srate}    = int($h->{FREQUENCY}*1000);
 	$rh{time}     = int($h->{SECS}*1000);
 	$rh{fdesc}    = "MPEG ${$h}{VERSION} layer ${$h}{LAYER} file";
 	
-	$h = MP3::Info::get_mp3tag($file,1)     unless $flags->{'noIDv1'};  #Get the IDv1 tag
+	$h  = MP3::Info::get_mp3tag($file,1)    unless $flags->{'noIDv1'};  #Get the IDv1 tag
 	$hs = MP3::Info::get_mp3tag($file, 2,1) unless $flags->{'noIDv2'};  #Get the IDv2 tag
 	
 	
@@ -268,18 +268,18 @@ sub __is_mp3 {
 	my @songa = pss(getutf8($hs->{TRCK} || $hs->{TRK} || $h->{TRACKNUM}));
 	my @cda   = pss(getutf8($hs->{TPOS}));
 	
-	$rh{songs}    = int($songa[1]);
-	$rh{songnum} =  int($songa[0]);
-	$rh{cdnum}   =  int($cda[0]);
-	$rh{cds}    =   int($cda[1]);
-	$rh{year} =     getutf8($hs->{TYER} || $hs->{TYE} || $h->{YEAR}    || 0);
-	$rh{title} =    getutf8($hs->{TIT2} || $hs->{TT2} || $h->{TITLE}   || $cf || "Untitled");
-	$rh{album} =    getutf8($hs->{TALB} || $hs->{TAL} || $h->{ALBUM}   || "Unknown Album");
-	$rh{artist} =   getutf8($hs->{TPE1} || $hs->{TP1} || $hs->{TPE2} || $hs->{TP2} || $h->{ARTIST}  || "Unknown Artist");
-	$rh{genre} =    _get_genre( getutf8($hs->{TCON} || $hs->{TCO} || $h->{GENRE}   || "") );
-	$rh{comment} =  getutf8($hs->{COMM} || $hs->{COM} || $h->{COMMENT} || "");
-	$rh{composer} = getutf8($hs->{TCOM} || $hs->{TCM} || "");
-	$rh{playcount}= int(getutf8($hs->{PCNT} || $hs->{CNT})) || 0;
+	$rh{songs}      = int($songa[1]);
+	$rh{songnum}    = int($songa[0]);
+	$rh{cdnum}      = int($cda[0]);
+	$rh{cds}        = int($cda[1]);
+	$rh{year}       = getutf8($hs->{TYER} || $hs->{TYE} || $h->{YEAR}    || 0);
+	$rh{title}      = getutf8($hs->{TIT2} || $hs->{TT2} || $h->{TITLE}   || $cf || "Untitled");
+	$rh{album}      = getutf8($hs->{TALB} || $hs->{TAL} || $h->{ALBUM}   || "Unknown Album");
+	$rh{artist}     = getutf8($hs->{TPE1} || $hs->{TP1} || $hs->{TPE2} || $hs->{TP2} || $h->{ARTIST}  || "Unknown Artist");
+	$rh{genre}      = _get_genre( getutf8($hs->{TCON} || $hs->{TCO} || $h->{GENRE}   || "") );
+	$rh{comment}    = getutf8($hs->{COMM} || $hs->{COM} || $h->{COMMENT} || "");
+	$rh{composer}   = getutf8($hs->{TCOM} || $hs->{TCM} || "");
+	$rh{playcount}  = int(getutf8($hs->{PCNT} || $hs->{CNT})) || 0;
 	$rh{soundcheck} = _parse_iTunNORM(getutf8($hs->{COMM} || $hs->{COM} || $h->{COMMENT}));
 	$rh{mediatype}  = MEDIATYPE_AUDIO;
 
@@ -339,50 +339,51 @@ sub pss {
 #########
 # Try to 'auto-guess' charset and return utf8
 sub getutf8 {
- my($in) = @_;
-
- return undef unless $in; #Do not fsckup empty input
-
- #Get the ENCODING
- $in =~ s/^(.)//;
- my $encoding = $1;
-
- # -> UTF16 with or without BOM
- if(ord($encoding) == 1 || ord($encoding) == 2) {
-  my $bfx = Unicode::String::utf16($in); #Object is utf16
-  $bfx->byteswap if $bfx->ord == 0xFFFE;
-  $in = $bfx->utf8; #Return utf8 version
- }
- # -> UTF8
- elsif(ord($encoding) == 3) {
-  my $bfx = Unicode::String::utf8($in)->utf8; #Paranoia
-  $in = $bfx;
- }
- # -> INVALID
- elsif(ord($encoding) > 0 && ord($encoding) < 32) {
-   warn "FileMagic.pm: warning: unsupportet ID3 Encoding found: ".ord($encoding)."\n";
-   warn "                       send a bugreport to pab\@blinkenlights.ch\n";
-   return undef;
- }
- # -> 0 or nothing
- else {
-  $in = $encoding.$in;
-  #Remove all 00's
-  $in =~ tr/\0//d;
-  my $oldstderr = *STDERR; #Kill all utf8 warnings.. this is uuugly
-  *STDERR = "NULLFH";
-  my $bfx = Unicode::String::utf8($in)->utf8;
-  *STDERR = $oldstderr;    #Restore old filehandle
-   if($bfx ne $in) {
-    #Input was no valid utf8, assume latin1 input
-    $in =~  s/[\000-\037]//gm; #Kill stupid chars..
-    $in = Unicode::String::latin1($in)->utf8
-   }
-   else { #Return the unicoded input
-    $in = $bfx;
-   }
- }
- return $in;
+	my($in) = @_;
+	return undef unless $in; #Do not fsckup empty input
+	
+	#Get the ENCODING
+	$in =~ s/^(.)//;
+	my $encoding = $1;
+	
+	# -> UTF16 with or without BOM
+	if(ord($encoding) == 1 || ord($encoding) == 2) {
+		# -> UTF16 with or without BOM
+		my $bfx = Unicode::String::utf16($in); #Object is utf16
+		$bfx->byteswap if $bfx->ord == 0xFFFE;
+		$in = $bfx->utf8; #Return utf8 version
+		if(unpack("H*",substr($in,0,3)) eq 'efbbbf') {
+			# -> Remove faulty utf16-to-utf8 BOM
+			$in = substr($in,3);
+		}
+	}
+	elsif(ord($encoding) == 3) {
+		# -> UTF8
+		$in = Unicode::String::utf8($in)->utf8; #Paranoia
+	}
+	elsif(ord($encoding) > 0 && ord($encoding) < 32) {
+		warn "FileMagic.pm: warning: unsupportet ID3 Encoding found: ".ord($encoding)."\n";
+		warn "                       send a bugreport to adrian\@blinkenlights.ch\n";
+		return undef;
+	}
+	else {
+		$in = $encoding.$in;     # Restores input
+		$in =~ tr/\0//d;         # Removes evil 0's
+		my $oldstderr = *STDERR; #Kill all utf8 warnings.. this is uuugly
+		*STDERR = "NULLFH";
+		my $bfx = Unicode::String::utf8($in)->utf8;
+		*STDERR = $oldstderr;    #Restore old filehandle
+		if($bfx ne $in) {
+			#Input was no valid utf8, assume latin1 input
+			$in =~  s/[\000-\037]//gm; #Kill stupid chars..
+			$in = Unicode::String::latin1($in)->utf8
+		}
+		else { #Return the unicoded input
+			$in = $bfx;
+		}
+	}
+	
+	return $in;
 }
 
 ##############################
@@ -399,21 +400,21 @@ sub _parse_iTunNORM {
 #########################################################
 # Start the converter
 sub kick_convert {
- my($prog, $quality, $file, $format, $con) = @_;
+	my($prog, $quality, $file, $format, $con) = @_;
 
- $prog = "$con->{bindir}/$prog";
- #Set Quality to a normal level
- $quality = 0 if $quality < 0;
- $quality = 9 if $quality > 9;
- open(KICKOMATIC, "-|") or exec($prog, $file, "GET_$format", int($quality)) or die "FileMagic::kick_convert: Could not exec $prog\n";
-  my $newP = <KICKOMATIC>;
-  chomp($newP);
- close(KICKOMATIC);
- 
- if($newP =~ /^PATH:(.+)$/) {
-  return $1;
- }
- return undef;
+	$prog = "$con->{bindir}/$prog";
+	#Set Quality to a normal level
+	$quality = 0 if $quality < 0;
+	$quality = 9 if $quality > 9;
+	open(KICKOMATIC, "-|") or exec($prog, $file, "GET_$format", int($quality)) or die "FileMagic::kick_convert: Could not exec $prog\n";
+	my $newP = <KICKOMATIC>;
+	chomp($newP);
+	close(KICKOMATIC);
+	
+	if($newP =~ /^PATH:(.+)$/) {
+		return $1;
+	}
+	return undef;
 }
 
 #########################################################
@@ -428,22 +429,15 @@ sub kick_reencode {
 	return undef if $quality > 9; #=Bad Quality
 	
 	#Try to get an unique name
-	my $tmpout = undef;
-	my $looppanic = 0;
-	while($tmpout = "/tmp/gnupod_re_".$$.int(rand(0xFFFF))) {
-		last unless -e $tmpout;
-		if($looppanic++ > 0xFFFF) {
-			warn "FileMagic.pm: Ouch: could not get an unique filename in /tmp .. \n";
-			return undef;
-		}
-	}
+	my $tmpout = GNUpod::FooBar::get_u_path("/tmp/gnupod_reencode", "tmp") or return undef;
 	
 	if($format eq 'm4a') {
 		#Faac is not as nice as lame: We have to decode ourself.. and fixup the $quality value
 		$quality = 140 - ($quality*10);
 		my $pcmout = $tmpout.".wav";
-		my $ret = system( ("faad", "-o", $pcmout, $file) );
+		system( ("faad", "-o", $pcmout, $file) );
 		#Ok, we've got a pcm version.. encode it!
+		
 		$tmpout .= ".m4a"; #Fixme: This breaks m4b.. well.. next time i'll fix it..
 		my $ret = system( ("faac", "-w", "-q", $quality, "-o", $tmpout, $pcmout) );
 		unlink($pcmout) or warn "FileMagic.pm: Could not unlink '$pcmout' , $!\n";
@@ -481,7 +475,7 @@ sub converter_readmeta {
 	while(<CFLAC>) {
 		chomp($_);
 		if($_ =~ /^([^:]+):(.*)$/) {
-		$metastuff{$1} = $2;
+			$metastuff{$1} = $2;
 		}
 	}
 	close(CFLAC);
