@@ -25,10 +25,10 @@ use strict;
 use GNUpod::XMLhelper;
 use GNUpod::FooBar;
 use GNUpod::FileMagic;
+use GNUpod::ArtworkDB;
 use Getopt::Long;
 use File::Copy;
 use File::Glob ':glob';
-use XML::Parser; #Loaded by XMLhelper, but hey..
 
 use constant MEDIATYPE_PODCAST_AUDIO => 4;
 use constant MEDIATYPE_PODCAST_VIDEO => 6;
@@ -44,10 +44,10 @@ $opts{mount} = $ENV{IPOD_MOUNTPOINT};
 #Don't add xml and itunes opts.. we *NEED* the mount opt to be set..
 GetOptions(\%opts, "version", "help|h", "mount|m=s", "decode=s", "restore|r", "duplicate|d", "disable-v2", "disable-v1",
                    "set-title=s", "set-artist=s", "set-album=s", "set-genre=s", "set-rating=i", "set-playcount=i",
-                   "set-bookmarkable", "set-shuffleskip",
+                   "set-bookmarkable", "set-shuffleskip", "artwork=s",
                    "set-songnum", "playlist|p=s@", "reencode|e=i",
                    "min-vol-adj=i", "max-vol-adj=i", "playlist-is-podcast", "set-compilation");
-GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b',
+GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b', model=>'s',
                                    'disable-v1'=>'b', 'disable-v2'=>'b', 'set-songnum'=>'b',
                                    'min-vol-adj'=>'i', 'max-vol-adj'=>'i', 'automktunes'=>'b' },
                                    "gnupod_addsong");
@@ -55,20 +55,16 @@ GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b',
 
 
 usage("\n--decode needs 'pcm' 'mp3' 'aac' 'video' or 'aacbm' -> '--decode=mp3'\n") if $opts{decode} && $opts{decode} !~ /^(mp3|video|aac|aacbm|pcm|crashme)$/;
-usage() if $opts{help};
+usage()   if $opts{help};
 version() if $opts{version};
 
 $SIG{'INT'} = \&handle_int;
+my @XFILES  = ();
 
-
-my @XFILES = ();
 if($opts{restore}) {
-	print "If you use --restore, you'll *lose* your playlists\n";
+	print "If you use --restore, you'll *lose* your playlists and cover artwork!\n";
 	print " Hit ENTER to continue or CTRL+C to abort\n\n";
 	<STDIN>;
-	delete($opts{decode});    #We don't decode anything
-	$opts{duplicate} = 1;     #Don't skip dups on restore
-	$opts{decode}    = undef; #Do not encode, only native files are on an iPod
 	@XFILES = bsd_glob("$opts{mount}/iPod_Control/Music/*/*", GLOB_NOSORT)
 }
 elsif($ARGV[0] eq "-" && @ARGV == 1) {
@@ -83,7 +79,13 @@ else {
 }
 
 
-my $exit_code = startup(@XFILES);
+my $connection = GNUpod::FooBar::connect(\%opts);
+usage($connection->{status}."\n") if $connection->{status} || !@XFILES;
+
+my $AWDB  = GNUpod::ArtworkDB->new(Connection=>$connection, DropUnseen=>1);
+
+
+my $exit_code = startup($connection,@XFILES);
 exit($exit_code);
 
 
@@ -93,26 +95,40 @@ exit($exit_code);
 ####################################################
 # Worker
 sub startup {
-	my(@argv_files) = @_;
+	my($con,@argv_files) = @_;
 	
 	#Don't sync if restore is true
 	$opts{_no_sync} = $opts{restore};
-	
 	my $fatal_error = 0;
-	my $con = GNUpod::FooBar::connect(\%opts);
-	usage($con->{status}."\n") if $con->{status} || !@argv_files;
-
-	unless($opts{restore}) { #We parse the old file, if we are NOT restoring the iPod
+	
+	if($opts{restore}) {
+		# Some options don't mix well with --restore
+		delete($opts{artwork});
+		delete($opts{playlist});
+		delete($opts{decode});
+		$opts{duplicate} = 1;
+	}
+	else {
+		if($opts{artwork}) {
+			if( $AWDB->PrepareImage(File=>$opts{artwork}, Model=>$opts{model}) ) {
+				$AWDB->LoadArtworkDb or die "Failed to load artwork database\n";
+			}
+			else {
+				warn "$0: Could not load $opts{artwork}, skipping artwork\n";
+				delete($opts{artwork});
+			}
+		}
 		GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}, did you run gnupod_INIT.pl?\n");
 	}
-
+	
 	if($opts{playlist}) { #Create this playlist
 		foreach my $xcpl (@{$opts{playlist}}) {
 			print "> Adding songs to Playlist '$xcpl'\n";
 			GNUpod::XMLhelper::addpl($xcpl, {podcast=>$opts{'playlist-is-podcast'}}); #Fixme: this may printout a warning..
 		}
 	}
-
+	
+	
 	# Check volume adjustment options for sanity
 	my $min_vol_adj = int($opts{'min-vol-adj'});
 	my $max_vol_adj = int($opts{'max-vol-adj'});
@@ -124,11 +140,9 @@ sub startup {
 	
 	#We parsed the XML-Document
 	#resolve_podcasts fetches new podcasts from http:// stuff and adds them to real_files
-#	warn "DEBUG: START RESOLVE\n";
 	my @real_files = resolve_podcasts(@argv_files);
-#	warn "DEBUG: END RESOLVE\n";
+	my $addcount   = 0;
 	
-	my $addcount = 0;
 	#We are ready to copy each file..
 	foreach my $file (@real_files) {
 		#Skip all songs if user sent INT
@@ -140,15 +154,16 @@ sub startup {
 		my ($fh,$media_h,$converter) =  GNUpod::FileMagic::wtf_is($file, {noIDv1=>$opts{'disable-v1'}, 
 		                                                                  noIDv2=>$opts{'disable-v2'},
 		                                                                  decode=>$opts{'decode'}},$con);
-
+		
 		unless($fh) {
 			warn "* [****] Skipping '$file', unknown file type\n";
 			next;
 		}
+		
 		my $wtf_ftyp = $media_h->{ftyp};      #'codec' .. maybe ALAC
 		my $wtf_frmt = $media_h->{format};    #container ..maybe M4A
 		my $wtf_ext  = $media_h->{extension}; #Possible extensions (regexp!)
-
+		
 		#Force tags for current file
 		#This is only used for RSS ATM.
 		my $c_per_file_info = $per_file_info{$file};
@@ -156,16 +171,8 @@ sub startup {
 			next unless lc($_) eq $_; #lc keys are there to overwrite $fh keys
 			$fh->{$_} = $c_per_file_info->{$_};
 		}
+		$c_per_file_info->{ISPODCAST} ||= $opts{'playlist-is-podcast'};  # Enforce podcast settings if we are going to create a pc-playlist
 		
-		# If this was a podcast, we need to fixup the mediatype
-		if($c_per_file_info->{ISPODCAST}) {
-			if($fh->{mediatype} == GNUpod::FileMagic::MEDIATYPE_AUDIO) {
-				$fh->{mediatype} = MEDIATYPE_PODCAST_AUDIO;
-			}
-			elsif($fh->{mediatype} == GNUpod::FileMagic::MEDIATYPE_VIDEO) {
-				$fh->{mediatype} = MEDIATYPE_PODCAST_VIDEO;
-			}
-		}
 		
 		#wtf_is found a filetype, override data if needed
 		$fh->{artist}       = $opts{'set-artist'}      if $opts{'set-artist'};
@@ -178,6 +185,11 @@ sub startup {
 		$fh->{playcount}    = $opts{'set-playcount'}   if $opts{'set-playcount'};
 		$fh->{title}        = $opts{'set-title'}       if $opts{'set-title'};
 		$fh->{songnum}      = 1+$addcount              if $opts{'set-songnum'};
+		if(defined($opts{artwork})) {
+			$fh->{has_artwork} = 1;
+			$fh->{artworkcnt}  = 1;
+			$fh->{dbid_1}      = $AWDB->InjectImage;
+		}
 		
 		#Set the addtime to unixtime(now)+MACTIME (the iPod uses mactime)
 		#This breaks perl < 5.8 if we don't use int(time()) !
@@ -194,37 +206,54 @@ sub startup {
 
 		#Check for duplicates
 		if(!$opts{duplicate} && (my $dup = checkdup($fh,$converter))) {
-			print "! [!!!] '$file' is a duplicate of song $dup, skipping file\n";
+			print "! [!!!!] '$file' is a duplicate of song $dup, skipping file\n";
 			create_playlist_now($opts{playlist}, $dup); #We also add duplicates to a playlist..
 			next;
 		}
-
+		
+		# If this was a podcast, we need to fixup the mediatype
+		if($c_per_file_info->{ISPODCAST}) {
+			$fh->{shuffleskip}   = 1;
+			$fh->{bookmarkable}  = 1;
+			$fh->{podcast}       = 1;
+			$fh->{podcastguid} ||= sprintf("GNUpodG%X",int(rand(0xFFFFF)));
+			$fh->{podcastrss}  ||= sprintf("GNUpodR%X",int(rand(0xFFFFF)));
+			
+			if($fh->{mediatype} == GNUpod::FileMagic::MEDIATYPE_AUDIO) {
+				$fh->{mediatype} = MEDIATYPE_PODCAST_AUDIO;
+			}
+			elsif($fh->{mediatype} == GNUpod::FileMagic::MEDIATYPE_VIDEO) {
+				$fh->{mediatype} = MEDIATYPE_PODCAST_VIDEO;
+				$wtf_frmt = "m4v"; # Enforce M4V as extension
+				$wtf_ext  = '';    # no multiple choices, sorry
+			}
+		}
+		
 		if($converter) {
 			print "> Converting '$file' from $wtf_ftyp into $opts{decode}, please wait...\n";
 			my $path_of_converted_file = GNUpod::FileMagic::kick_convert($converter,$opts{reencode},$file, uc($opts{decode}), $con);
 			unless($path_of_converted_file) {
-				print "! [!!!] Could not convert $file into $opts{decode}\n";
+				print "! [!!!!] Could not convert $file into $opts{decode}\n";
 				next;
 			}
 			#Ok, we got a converted file, fillout the gaps
 			my($conv_fh, $conv_media_h) = GNUpod::FileMagic::wtf_is($path_of_converted_file, undef, $con);
 			
 			unless($conv_fh) {
-				warn "* [***] Internal problem: $converter did not produce valid data.\n";
-				warn "* [***] Something is wrong with $path_of_converted_file (file not deleted, debug it! :) )\n";
+				warn "* [****] Internal problem: $converter did not produce valid data.\n";
+				warn "* [****] Something is wrong with $path_of_converted_file (file not deleted, debug it! :-) )\n";
 				next; 	
 			}
-    
+			
 			#We didn't know things like 'filesize' before...
 			$fh->{time}     = $conv_fh->{time};
 			$fh->{bitrate}  = $conv_fh->{bitrate};
 			$fh->{srate}    = $conv_fh->{srate};        
 			$fh->{filesize} = $conv_fh->{filesize};   
-			$wtf_frmt = $conv_media_h->{format};    #Set the new format (-> container)
-			$wtf_ext  = $conv_media_h->{extension}; #Set the new possible extension
-			#BUT KEEP ftyp! (= codec)
-			$file = $path_of_converted_file; #Point $file to new file
-			$per_file_info{$file}->{UNLINK} = 1; #Request unlink of this file after adding
+			$wtf_frmt       = $conv_media_h->{format};    #Set the new format (-> container)
+			$wtf_ext        = $conv_media_h->{extension}; #Set the new possible extension, but keep ftype (=codec)
+			$file           = $path_of_converted_file;    #Point $file to new file
+			$per_file_info{$file}->{UNLINK} = 1;          #Request unlink of this file after adding
 		}
 		elsif(defined($opts{reencode})) {
 			print "> ReEncoding '$file' with quality ".int($opts{reencode}).", please wait...\n";
@@ -234,20 +263,18 @@ sub startup {
 				#Ok, we could convert.. check if it made sense:
 				if( (-s $path_of_converted_file) < (-s $file) ) {
 					#Ok, output is smaller, we are going to use thisone
-					#1. Replace path to file
-					$file = $path_of_converted_file;
-					#2. Set UNLINK-state : This will unlink the file after copy finished!
-					$per_file_info{$file}->{UNLINK} = 1;
+					$file = $path_of_converted_file;      # Replace the file path
+					$per_file_info{$file}->{UNLINK} = 1;  # Request unlinking file (as we are going to use the copy)
 				}
 				else {
 					#Nope.. input was smaller, converting was silly..
-					print "* [***] Reencoded output bigger than input! Adding source file\n";
+					print "* [****] Reencoded output bigger than input! Adding source file\n";
 					unlink($path_of_converted_file) or warn "Could not unlink $path_of_converted_file, $!\n";
 					#Ok, do nothing! 
 				}
 			}
 			else {
-				print "* [***] ReEncoding of file failed! Adding given file\n";
+				print "* [****] ReEncoding of file failed! Adding given file\n";
 			}
 		}
 		
@@ -255,13 +282,11 @@ sub startup {
 		my $vol = $fh->{volume} || 0;
 		$vol = $min_vol_adj if ($vol < $min_vol_adj);
 		$vol = $max_vol_adj if ($vol > $max_vol_adj);
-		# print "$file vol $fh->{volume} -> $vol\n";
 		$fh->{volume} = $vol;
 		
 		#Get a path
-		(${$fh}{path}, my $target) = GNUpod::XMLhelper::getpath($opts{mount}, $file, 
-		                                                        {format=>$wtf_frmt, extension=>$wtf_ext, keepfile=>$opts{restore}});
-
+		($fh->{path}, my $target) = GNUpod::XMLhelper::getpath($opts{mount}, $file,  {format=>$wtf_frmt, extension=>$wtf_ext, keepfile=>$opts{restore}});
+		
 		if(!defined($target)) {
 			warn "*** FATAL *** Skipping '$file' , no target found!\n";
 			$fatal_error++;
@@ -296,6 +321,7 @@ sub startup {
 		print "> Writing new XML File, added $addcount file(s)\n";
 		GNUpod::XMLhelper::writexml($con, {automktunes=>$opts{automktunes}});
 	}
+	$AWDB->WriteArtworkDb;
 	print "\n Done\n";
 	return $fatal_error;
 }
@@ -337,12 +363,12 @@ sub newfile {
 	if($_[0]->{file}->{podcastguid}) {
 		$dupdb_podcast{$_[0]->{file}->{podcastguid}."\0".$_[0]->{file}->{podcastrss}}++;
 	}
-	
+	$AWDB->KeepImage($_[0]->{file}->{dbid_1});
 	GNUpod::XMLhelper::mkfile($_[0],{addid=>1});
 }
 
 sub newpl {
- GNUpod::XMLhelper::mkfile($_[0],{$_[2]."name"=>$_[1]});
+	GNUpod::XMLhelper::mkfile($_[0],{$_[2]."name"=>$_[1]});
 }
 ##################
 
@@ -416,7 +442,7 @@ sub resolve_podcasts {
 			
 			my $pcrss = PODCAST_fetch($cf, "/tmp/gnupodcast$i");
 			if($pcrss->{status} or (!(-f $pcrss->{file}))) {
-				warn "! [HTTP] Unable to download the file '$cf', wget exitcode: $pcrss->{status}\n";
+				warn "! [HTTP] Failed to download the file '$cf', wget exitcode: $pcrss->{status}\n";
 				next;
 			}
 			#Add the stuff to %podcast_infos and unlink the file after this.
@@ -433,52 +459,47 @@ sub resolve_podcasts {
 		}
 	}
 
-foreach my $key (keys(%podcast_infos)) {
-	my $cref = $podcast_infos{$key};
-	foreach my $podcast_item (@$cref) {
-		my $c_title = $podcast_item->{title}->{"\0"};
-		my $c_author = $podcast_item->{author}->{"\0"};
-		my $c_desc  = $podcast_item->{description}->{"\0"};
-		my $c_url   = $podcast_item->{enclosure}->{url};
-		#We use the URL as GUID if there isn't one...			
-		my $c_guid  = $podcast_item->{guid}->{"\0"} || $c_url;
-		my $c_podcastrss = $per_file_info{$key}->{REAL_RSS};
-		my $possible_dupdb_entry = $c_guid."\0".$c_podcastrss;
-
-		if(length($c_guid) == 0 or length($c_podcastrss) == 0 or length($c_url) == 0) {
-			warn "! [HTTP] '$c_podcastrss' is an invalid podcast item (No URL/RSS?)\n";
-			next;
+	foreach my $key (keys(%podcast_infos)) {
+		my $cref = $podcast_infos{$key};
+		foreach my $podcast_item (@$cref) {
+			my $c_title = $podcast_item->{title}->{"\0"};
+			my $c_author = $podcast_item->{author}->{"\0"};
+			my $c_desc  = $podcast_item->{description}->{"\0"};
+			my $c_url   = $podcast_item->{enclosure}->{url};
+			#We use the URL as GUID if there isn't one...			
+			my $c_guid  = $podcast_item->{guid}->{"\0"} || $c_url;
+			my $c_podcastrss = $per_file_info{$key}->{REAL_RSS};
+			my $possible_dupdb_entry = $c_guid."\0".$c_podcastrss;
+			
+			if(length($c_guid) == 0 or length($c_podcastrss) == 0 or length($c_url) == 0) {
+				warn "! [HTTP] '$c_podcastrss' is an invalid podcast item (No URL/RSS?)\n";
+				next;
+			}
+			elsif($dupdb_podcast{$possible_dupdb_entry}) {
+				warn "! [HTTP] Podcast $c_url ($c_title) exists, no need to download this file\n";
+				next;
+			}		
+			print "* [HTTP] Downloading $c_url ...\n";
+			my $rssmedia = PODCAST_fetch($c_url, "/tmp/gnupodcast_media");
+			if($rssmedia->{status} or (!(-f $rssmedia->{file}))) {
+				warn "! [HTTP] Failed to download $c_url to $rssmedia->{file}\n";
+				next;
+			}
+			
+			$per_file_info{$rssmedia->{file}}->{UNLINK}    = 1;  # Remote tempfile
+			$per_file_info{$rssmedia->{file}}->{ISPODCAST} = 1;  # Triggers mediatype fix
+			
+			# Set information/tags from XML-File
+			$per_file_info{$rssmedia->{file}}->{podcastguid} = $c_guid;
+			$per_file_info{$rssmedia->{file}}->{podcastrss}  = $c_podcastrss;
+			$per_file_info{$rssmedia->{file}}->{title}       = $c_title   if $c_title;
+			$per_file_info{$rssmedia->{file}}->{artist}      = $c_author  if $c_author;
+			$per_file_info{$rssmedia->{file}}->{desc}        = $c_desc    if $c_desc;
+			
+			push(@files,$rssmedia->{file});
 		}
-		elsif($dupdb_podcast{$possible_dupdb_entry}) {
-			warn "! [HTTP] Podcast $c_url ($c_title) exists, no need to download this file\n";
-			next;
-		}		
-		print "* [HTTP] Downloading $c_url ...\n";
-		my $rssmedia = PODCAST_fetch($c_url, "/tmp/gnupodcast_media");
-		if($rssmedia->{status} or (!(-f $rssmedia->{file}))) {
-			warn "! [HTTP] Unable to download $rssmedia->{file}\n";
-			next;
-		}
-
-		$per_file_info{$rssmedia->{file}}->{UNLINK}    = 1;  # Remote tempfile
-		$per_file_info{$rssmedia->{file}}->{ISPODCAST} = 1;  # Triggers mediatype fix
-		
-		# Set information/tags from XML-File
-		$per_file_info{$rssmedia->{file}}->{podcastguid} = $c_guid;
-		$per_file_info{$rssmedia->{file}}->{podcastrss}  = $c_podcastrss;
-		$per_file_info{$rssmedia->{file}}->{title}       = $c_title   if $c_title;
-		$per_file_info{$rssmedia->{file}}->{artist}      = $c_author  if $c_author;
-		$per_file_info{$rssmedia->{file}}->{desc}        = $c_desc    if $c_desc;
-		
-		# Do the same as iTunes does:
-		$per_file_info{$rssmedia->{file}}->{shuffleskip}  = 1;
-		$per_file_info{$rssmedia->{file}}->{bookmarkable} = 1;
-		$per_file_info{$rssmedia->{file}}->{podcast}      = 1;
-		
-		
-		push(@files,$rssmedia->{file});
 	}
-}
+	
 	return @files;
 }
 
@@ -547,6 +568,8 @@ Usage: gnupod_addsong.pl [-h] [-m directory] File1 File2 ...
        --max-vol-adj=int            Maximum ditto.  The volume can be adjusted in the range
                                     -100% to +100%.  The default for these two options is 0,
                                     which effectively ignored the RVA2 tag.
+       --artwork=FILE               Use FILE as album cover
+
 
 Report bugs to <bug-gnupod\@nongnu.org>
 EOF
@@ -555,7 +578,7 @@ EOF
 sub version {
 die << "EOF";
 gnupod_addsong.pl (gnupod) ###__VERSION__###
-Copyright (C) Adrian Ulrich 2002-2005
+Copyright (C) Adrian Ulrich 2002-2007
 
 This is free software; see the source for copying conditions.  There is NO
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
