@@ -25,19 +25,18 @@ use strict;
 use GNUpod::XMLhelper;
 use GNUpod::FooBar;
 use GNUpod::iTunesDB;
-use GNUpod::lastfm;
 use Getopt::Long;
 
 use File::Glob ':glob';
 
-use vars qw(%opts @keeper $plcref %lastfm_data $lastfm_timezone_hack);
+use vars qw(%opts @keeper $plcref);
 
 
 $opts{mount} = $ENV{IPOD_MOUNTPOINT};
 
 #Don't add xml and itunes opts.. we *NEED* the mount opt to be set..
 GetOptions(\%opts, "top4secret");
-GNUpod::FooBar::GetConfig(\%opts, {nosync=>'b', lastfm_enabled=>'b', lastfm_user=>'s', lastfm_password=>'s', 'automktunes'=>'b'}, "otgsync");
+GNUpod::FooBar::GetConfig(\%opts, {nosync=>'b', 'automktunes'=>'b'}, "otgsync");
 #otgsync does just red nosync.. DONT add mount and such funny things!
 
 
@@ -58,28 +57,22 @@ else {
 # Worker
 sub go {
 
-	#Disable auto-run of tunes2pod or gnupod_otgsync.pl
-	$opts{_no_sync} = 1;
+	$opts{_no_sync}   = 1;  # Do not run tunes2pod
+	$opts{_no_cstest} = 1;  # Do not show case-sensitive warning
 	my $con = GNUpod::FooBar::connect(\%opts);
 	usage($con->{status}."\n") if $con->{status};
 	
-	if(GNUpod::FooBar::_itb_needs_sync($con)) {
+	if(GNUpod::FooBar::ItunesDBNeedsSync($con)) {
 		die "gnupod_otgsync.pl: Bug detected! You need to run tunes2pod.pl -> Sync broken!\n";
 	}
 	
 	##Check if GNUtunesDB <-> iTunesDB is really in-sync
-	if(GNUpod::FooBar::_otgdata_broken($con)) { #Ok, On-The-Go data is ** BROKEN **
-		warn "gnupod_otgsync.pl: Error: You forgot to run mktunes.pl, wiping broken On-The-Go data...\n";
-		#Remove broken data.. live is hard..
-		unlink(bsd_glob($con->{onthego}, GLOB_NOSORT)) or warn "Could not remove $con->{onthego}, $!\n";
-		unlink($con->{playcounts}) or warn "Could not remove $con->{playcounts}, $!\n"; 
-		warn "Done!\n";
+	if(GNUpod::FooBar::OnTheGoDataIsInvalid($con)) { #Ok, On-The-Go data is ** BROKEN **
+		warn "> On-The-Go data dropped, data belongs to an outdated iTunesDB\n";
 	}
 	else {
-		$lastfm_timezone_hack = -1* $con->{tzdiff};
 		#Read on The Go list written by the iPod
 		my @xotg    = GNUpod::iTunesDB::readOTG($con->{onthego});
-		
 		#plcref is used by newfile()
 		#so we have to call this before doxml()
 		$plcref  = GNUpod::iTunesDB::readPLC($con->{playcounts});
@@ -89,56 +82,13 @@ sub go {
 			#First, we parse the old xml document and create the keeper
 			GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}\n");
 			mkotg(@xotg) if int(@xotg);
-			GNUpod::FooBar::setsync($con); # Needed for automktunes
+			GNUpod::FooBar::SetEverythingAsInSync($con); # Needed for automktunes
 			GNUpod::XMLhelper::writexml($con, {automktunes=>$opts{automktunes}});
 		}
-		else {
-			#setsync .. just to be sure..
-			GNUpod::FooBar::setsync($con);
-		}
-		#..and submit lastfm data if enabled in config file
-
-		lfmworker($con->{lastfm_queue}) if $opts{lastfm_enabled}
-	}
-}
-
-sub lfmsubmit {
-	my($fm) = @_;
-	return GNUpod::lastfm::dosubmission({user=>$opts{lastfm_user}, password=>$opts{lastfm_password}, tosubmit=>$fm});
-}
-
-
-sub lfmworker {
-	my($queue) = @_;
-	
-	if(-e $queue) {
-		print "> lastfm: Uploading data from lastfm queue ($queue)\n";
-		open(LFMQ, $queue) or die "Ouch: Could not read $queue: $!\n";
-		my $lfmq = lfmsubmit(GNUpod::lastfm::simple_lastfm_restore(*LFMQ));
-		close(LFMQ);
-		if($lfmq) {
-			warn "> lastfm: Upload of queued data failed!\n";
-			warn "> lastfm: Delete '$queue' if the problem persists.\n";
-		}
-		else {
-			print "> lastfm: Uploaded queued data!\n";
-			unlink($queue) or warn "Could not unlink $queue : $!\n";
-		}
 	}
 	
-	#Work on lastfm data.. %lastfm_data is empty if feature disabled
-	my @fm = ();
-	foreach(sort keys(%lastfm_data)) {
-		push(@fm, $lastfm_data{$_});
-	}
-	my $lfmstat = lfmsubmit(\@fm);
-	
-	if($lfmstat) {
-		warn "> lastfm: Upload failed, dumping data to queue\n";
-		open(LFMQ, ">>".$queue) or die "Could not write to $queue: $!\n";
-		GNUpod::lastfm::simple_lastfm_dump(*LFMQ,\@fm);
-		close(LFMQ);
-	}
+	# Everythig should be ok now..
+	GNUpod::FooBar::SetEverythingAsInSync($con);
 }
 
 
@@ -200,30 +150,6 @@ sub newfile {
 		if($plcref->{lastskip}{int(@keeper)-1}) {
 			$el->{file}->{lastskip}  = $plcref->{lastskip}{int(@keeper)-1};
 		}
-		
-		if($playcount > 0 && $opts{lastfm_enabled}) {
-			my $seconds = int($el->{file}->{time}/1000);
-			for(1..$playcount) {
-				#Fixme: We (currently) do not care about the Timezone on the iPod
-				my $gmtime = $el->{file}->{lastplay} - GNUpod::FooBar::MACTIME + $lastfm_timezone_hack; #fixme: this may cause collisions
-				my @gmt    = gmtime($gmtime);
-				my $lfmtime = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $gmt[5]+1900, $gmt[4]+1, $gmt[3], $gmt[2], $gmt[1], $gmt[0]);
-				my $steptime = int($gmtime/10); #10-seconds 'resulution'
-				
-				#Search a free place for this song somewhere in our queue.
-				#This is needed because the song may be played multiple times but we got only
-				#the latest playtime from the iPod's database.
-				while($lastfm_data{$steptime}) {
-					$steptime--; #= 10 seconds
-					print "=> Ouch! Adjusting Steptime to $steptime for ".$el->{file}->{title}."\n";
-				}
-				
-				$lastfm_data{$steptime} =   {artist => $el->{file}->{artist}, album => $el->{file}->{album}, 
-				                             title => $el->{file}->{title}, length => $el->{file}->{time}, xplaydate=>$lfmtime};
-				print "LASTFM:    => $lfmtime   $el->{file}->{title} $el->{file}->{lastplay} $el->{file}->{time}\n";
-			}
-		}
-		
 	}
 	#Add content
 	GNUpod::XMLhelper::mkfile($el);
