@@ -27,6 +27,7 @@ use GNUpod::FooBar;
 use GNUpod::Mktunes;
 use GNUpod::Hash58;
 use GNUpod::SysInfo;
+use GNUpod::ArtworkDB;
 use Getopt::Long;
 
 use constant MPL_UID => 1234567890; #This is the MasterPlaylist ID
@@ -58,13 +59,17 @@ sub main {
 	my $sysinfo = GNUpod::SysInfo::GetDeviceInformation(Connection=>$con, NoDeviceSearch=>(defined($opts{fwguid}) ? 1 : 0 ) );
 	my $fwguid  = (defined($opts{fwguid}) ? $opts{fwguid} : $sysinfo->{FirewireGuid}); # Always prefer fwguid. may be 0 to disable search
 	
+	print "> Loading ArtworkDB...";
+	my $AWDB  = GNUpod::ArtworkDB->new(Connection=>$con, DropUnseen=>0);
+	$AWDB->LoadArtworkDb;
+	print "done\n";
 	
-	$mktunes = GNUpod::Mktunes->new(Connection=>$con, iPodName=>$opts{'ipod-name'});
+	$mktunes = GNUpod::Mktunes->new(Connection=>$con, iPodName=>$opts{'ipod-name'}, Artwork=>$AWDB);
 	
 	print "> Parsing XML document...\n";
 	GNUpod::XMLhelper::doxml($con->{xml}) or usage("Could not read $con->{xml}, did you run gnupod_INIT.pl ?");
 	
-	print "\r> ".$mktunes->GetFileCount." files parsed, writing new iTunesDB...";
+	print "\r> ".$mktunes->GetFileCount." files parsed, assembling iTunesDB...\n";
 	$mktunes->WriteItunesDB;
 	
 	if($fwguid) {
@@ -78,9 +83,11 @@ sub main {
 	$mktunes->WriteItunesSD;
 	
 	print "> Updating Sync-Status\n";
-	GNUpod::FooBar::setsync_itunesdb($con);
-	GNUpod::FooBar::setvalid_otgdata($con);
-	GNUpod::FooBar::wipe_shufflestat($con);
+	GNUpod::FooBar::SetItunesDBAsInSync($con);   # iTunesDB is in sync with GNUtunesDB.xml
+	GNUpod::FooBar::SetOnTheGoAsValid($con);     # ..and we can now, again, trust OnTheGo data
+	GNUpod::FooBar::WipeShuffleStat($con);       # Forces reshuffling of iPod-Shuffle
+	print "\nYou can now umount your iPod. [Files: ".$mktunes->GetFileCount."]\n";
+	print " - May the iPod be with you!\n\n";
 
 }
 
